/********************************************************************************************************\

	File Name:	CommonTeam.cpp
	Descript:	<DESCRIPTION>

\********************************************************************************************************/

#include "tclient.h"
#include "MissionText.h"
#include "CommonBaseVictory.h"
#include "CommonDefs.h"
#include "CommonFrigate.h"
#include "CommonTeam.h"
#include "CommonTeamBaseState.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"
#include "Mul_3Battlefest.h"

tClassAllocator<tCommonTeam> gCommonTeamAllocator;


tCommonTeam::tCommonTeam( void )
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::tCommonTeam\n");
	fflush(fp);
	fclose(fp);
#endif

	gWC.fNumTeams++;
}

void tCommonTeam::mSetupVictoryCondition()
{

#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mSetupVictoryCondition\n");
	fflush(fp);
	fclose(fp);
#endif

	fVictoryCondition = new tCommonBaseVictory( this );
	gEventMgr.mRegisterActor( fVictoryCondition );
}

void tCommonTeam::mSetupDebriefingList( eVictoryLevels victoryLevel )
{

#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mSetupDebriefingList\n");
	fflush(fp);
	fclose(fp);
#endif

	switch ( victoryLevel )
	{
	case kAstoundingVictory:
	case kVictory:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kWinner_msg );
		}
		break;

	case kDefeat:
	case kDevastatingDefeat:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLoser_msg );
		}
		break;

	case kUnknownVictoryLevel:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kLeftEarly_msg );
		}
		break;

	case kDraw:
	default:
		{
			fMissionInfo->mAddToDebriefingList( fTeam, kDefault_msg );
		}
		break;
	}
}

/*

	HELPER NOTE:	If you create any new states on the team level, you must register them here first.

*/
void tCommonTeam::mSetupState()
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mSetupState\n");
	fflush(fp);
	fclose(fp);
#endif

	mRegisterState( new tCommonTeamBaseState() );
	mGotoState( typeid( tCommonTeamBaseState ) );
}

void tCommonTeam::mSetupCommState()
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mSetupCommState\n");
	fflush(fp);
	fclose(fp);
#endif
}

/*

	HELPER NOTE:	In this function, you will create the ships for the team described by the file name.
					It is a good idea to give each of these ships a custom ID so that it can be referred
					to later (i.e. victory condition).  The custom IDs are listed in the matching header file.
					The ships can be created in one of two ways.  There are examples of both below.  One last
					thing, when you create these ships, you have to remember to include their class headers at
					the top of this file; otherwise, this file will not compile.

	//
	//	CREATE SHIP #1: Creates a ship based on the player's BPV.
	//
	tTeamInfo* playerTeam = fMissionInfo->mGetTeamHandle( gScriptInterface.mPlayersTeamID() );

	int32 playerBPV = playerTeam->mGetCombatBPV();
	int32 diffLevel = fMissionInfo->mGetDifficultyLevel();

	int32 nPCBPV; 
	if( diffLevel == 0 )
	{
		nPCBPV = playerBPV * 0.75;
	}
	else if( diffLevel == 1 )
	{
		nPCBPV = playerBPV * 1.0;
	}
	else if( diffLevel == 2 )
	{
		nPCBPV = playerBPV * 1.25;
	}
	mCreateShip(	typeid( tPlayerNPCBS1 ),
					nPCBPV,
					kClassFrigate,
					kClassHeavyBattlecruiser,
					0,
					kStartPosition_Z,
					kPlayerNPCBS1 );

	//
	//	CREATE SHIP #2: Creates a ship based on one of the spec records found in the spec chart.
	//
	mCreateShip( typeid( tPlayerNPCBS1 ), "PL3" , kStartPosition_J, kPlayerNPCBS2 );

*/
void tCommonTeam::mCreateShipsForTeam()
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering tCommonTeam::mCreateShipsForTeam\n");
	fflush(fp);
#endif

	int32 Team = (int32)mGetTeam();
	int32 Race = (int32)mGetRace();

	// Create the AI teams now, before shiplist generation.
	if (!AIsCreated) {
		for (int humanTeam = 0; humanTeam < 6; humanTeam++)
		{
			tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)humanTeam);
			if (ti)
			{
				fMissionInfo->mSetupTeam( (eTeamID)humanTeam+6,
					ti->mGetTag(),
					ti->mGetRace(),
					kFalse, // NOT Human Controlled!
					typeid( tCommonTeam ) );
			}
		}
		AIsCreated = 1;
	}

#ifdef DEBUGOUT
	fprintf(fp, "CommonTeam for Team # %d\n", Team);
	fflush(fp);
#endif

	// proper era-detection code
	if (!ShipListComplete) {

#ifdef DEBUGOUT
		fprintf(fp, "** Narrowing shiplists\n");
		fflush(fp);
#endif

		// 1- prepare the era data.
		// mGetEra is in OP only
		int32 selected_era = fMissionInfo->mGetEra();
#ifdef DEBUGOUT
		fprintf(fp, "** Era selected: %d\n", selected_era);
		fflush(fp);
#endif

		int32 selected_era_end_year = 999;
		int32 selected_era_begin_year = -999;

		if (selected_era == kEarly)
		{
			selected_era_end_year = kEEnd;
			selected_era_begin_year = kEBegin;
		}
		else if (selected_era == kMiddle)
		{
			selected_era_end_year = kMEnd;
			selected_era_begin_year = kMBegin;
		}
		else if (selected_era == kLate)
		{
			selected_era_end_year = kLEnd;
			selected_era_begin_year = kLBegin;
		}
		else if (selected_era == kAdvanced)
		{
			selected_era_end_year = kAEnd;
			selected_era_begin_year = kABegin;
		}

		// Load ShipList
		if (global_sl.size() == 0) {
			global_sl.CreateMaster(ShipList::kOrionPirates, false);
		}

#ifdef FIGHTERS
		if (fighter_sl.size() == 0) {
			fighter_sl.CreateFighterList(ShipList::kOrionPirates, false);
		}
		narrowed_fighter_sl = fighter_sl;
#endif

		{
		SHIP_LIST_ITER(global_sl)
		int ship_removed = 0;
		int ship_removed_backup = 0;

		// backup sublist: ships in existance in current era.
		if (
			(iter->yearLastAvail < selected_era_begin_year) || 
			(iter->yearFirstAvail > selected_era_end_year)
			) {
			ship_removed_backup = 1;
		}

		// main sublist: ships in existance DURING current era
		if (
			!(
			(iter->yearFirstAvail >= selected_era_begin_year) &&
			(iter->yearFirstAvail <= selected_era_end_year)
			)
			) {
			ship_removed = 1;
		}

		// remove ship if YFA is 999 (tourneys)
		if (iter->yearFirstAvail == 999) {
			//narrowed_sl.erase(iter);
			ship_removed = 1;
		}
		

		// For each of YOUR team(s)
		//   if ship's race matches team's race
		//     add to appropriate sublist

		// sl_array[teams][races][classes]
		if (!ship_removed) {
			for (int team = 0; team <= 11; team++)
			{
				tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
				if (ti)
				{
					/*
					if (
						((ti->mGetTeam() == (eTeamID)Team) && ti->mIsTeamHumanControlled()) ||
						(!ti->mIsTeamHumanControlled() && gScriptInterface.mIsHost())
						)
					*/
					{
						eRace team_race = ti->mGetRace();
						eRace ship_race = iter->race;

						if (team_race == ship_race) {
							eClassTypes ship_class = iter->shipClass;

							if (ship_class == kClassFrigate) {
								sl_array[team][(int )team_race][0].push_back(*iter);
							}
							if ((ship_class == kClassDestroyer) || (ship_class == kClassWarDestroyer)) {
								sl_array[team][(int )team_race][1].push_back(*iter);
							}
							if (ship_class == kClassLightCruiser) {
								sl_array[team][(int )team_race][2].push_back(*iter);
							}
							if ((ship_class == kClassHeavyCruiser) || (ship_class == kClassNewHeavyCruiser)) {
								sl_array[team][(int )team_race][3].push_back(*iter);
							}
							if (ship_class == kClassHeavyBattlecruiser) {
								sl_array[team][(int )team_race][4].push_back(*iter);
							}
							if (ship_class == kClassDreadnought) {
								sl_array[team][(int )team_race][5].push_back(*iter);
							}
							if (ship_class == kClassBattleship) {
								sl_array[team][(int )team_race][6].push_back(*iter);
							}
						}
					}
				}
			}
		}

		// and for the backup lists.
		if (!ship_removed_backup) {
			for (int team = 0; team <= 11; team++)
			{
				tTeamInfo *ti = fMissionInfo->mGetTeamHandle((eTeamID)team);
				if (ti)
				{
					/*
					if (
						((ti->mGetTeam() == (eTeamID)Team) && ti->mIsTeamHumanControlled()) ||
						(!ti->mIsTeamHumanControlled() && gScriptInterface.mIsHost())
						)
					*/
					{
						eRace team_race = ti->mGetRace();
						eRace ship_race = iter->race;

						if (team_race == ship_race) {
							eClassTypes ship_class = iter->shipClass;

							if (ship_class == kClassFrigate) {
								sl_array_backup[team][(int )team_race][0].push_back(*iter);
							}
							if ((ship_class == kClassDestroyer) || (ship_class == kClassWarDestroyer)) {
								sl_array_backup[team][(int )team_race][1].push_back(*iter);
							}
							if (ship_class == kClassLightCruiser) {
								sl_array_backup[team][(int )team_race][2].push_back(*iter);
							}
							if ((ship_class == kClassHeavyCruiser) || (ship_class == kClassNewHeavyCruiser)) {
								sl_array_backup[team][(int )team_race][3].push_back(*iter);
							}
							if (ship_class == kClassHeavyBattlecruiser) {
								sl_array_backup[team][(int )team_race][4].push_back(*iter);
							}
							if (ship_class == kClassDreadnought) {
								sl_array_backup[team][(int )team_race][5].push_back(*iter);
							}
							if (ship_class == kClassBattleship) {
								sl_array_backup[team][(int )team_race][6].push_back(*iter);
							}
						}
					}
				}
			}
		}

		if (! ship_removed) {
			narrowed_sl.push_back(*iter);
		}

		SHIP_LIST_ITER_END
		}


#ifdef FIGHTERS
		// narrow the fighter list to fit the era.
		ShipList tmp_fighter_narrowed_sl = narrowed_fighter_sl;

		{
			SHIP_LIST_ITER(tmp_fighter_narrowed_sl)
			// main sublist: fighters in existance DURING current era
			if ((iter->yearLastAvail - 164 < selected_era_begin_year)
				|| (iter->yearFirstAvail > selected_era_end_year)) 
			{
				fighter_sl.Remove(iter->shipCode);
			} else if (iter->bpv < 0) {
				// We don't want to downgrade to something smaller than what paid.
				narrowed_fighter_sl.Remove(iter->shipCode);
			} else if (iter->geoUI == "XSH") {
				// get rid of possible shuttles
				narrowed_fighter_sl.Remove(iter->shipCode);
			}
			SHIP_LIST_ITER_END
		}
#endif

		ShipListComplete = 1;

#ifdef DEBUGOUT
		fprintf(fp, "** Done narrowing shiplists\n");
		fflush(fp);
#endif
	}


	/*
	// OLD ship creation call:
	*/


	// create the ship
	// Selecting FF class ship
#ifdef DEBUGOUT
	fprintf(fp, "Creating FF Class ship\n");
	fflush(fp);
#endif

	int classic_style = 0;
	ShipList tmpsl = sl_array[(int32 )mGetTeam()][(int32 )mGetRace()][0];

	// if there's no ship in the FF list, select from DD list.
	if (tmpsl.size() == 0) {
		// if there's no ship in the DD list either, ....
		tmpsl = sl_array[(int32 )mGetTeam()][(int32 )mGetRace()][1];
		if (tmpsl.size() == 0) {
			classic_style = 1;
		}
	}

	if (!classic_style) {
		Ship s;
		int result = tmpsl.SelectRandomShip(s);

//		fprintf(fp, "Creating ship: %s\n", s.shipCode.c_str());
//		fflush(fp);
		mCreateShip(	typeid( tCommonFrigate ),
					s.shipCode.c_str(),
					gWC.mGenerateRandomStartPosition(),
					kFrigate,
					-1, 0, 0, -1, -1, NULL,
					static_cast< eShipOptions >( kDefaultShipOptions | kDeathUponDisengage ) );
	} else {
		// classic way of selecting a ship (era insensitive)
			mCreateShip(	typeid( tCommonFrigate ),
					kMaxBPV,
					kClassFrigate,
					100,
					gWC.mGenerateRandomStartPosition(),
					kFrigate,
					0, 0, -1, -1, NULL,
					static_cast< eShipOptions >( kDefaultShipOptions | kDeathUponDisengage ) );

	}
#ifdef DEBUGOUT
	fprintf(fp, "Done creation: FF\n");
	fflush(fp);
	fclose(fp);
#endif
}

void tCommonTeam::mTeamSetupComplete()
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mTeamSetupComplete\n");
	fflush(fp);
	fclose(fp);
#endif

	tTeamInfo::mTeamSetupComplete();

	tShipIterator ship = mGetFirstShip();

	Assert( *ship != NULL );
	if ( *ship != NULL )
	{
		if ( mIsTeamHumanControlled() )
		{
			ship->mSetShipPlayerControlled();
		}
	}

	for( ; *ship != NULL; ++ship )
	{
		ship->mSetShipCallbackEvents( kNotifyDamage );
	}

	//
	//	Starting message for each team.
	//
	fMissionInfo->mDisplayMessage( fTeam, kStarting_msg );
}

void tCommonTeam::mCreateNewShip( const tShipScriptDescription& shipScriptDescription )
{

#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tCommonTeam::mCreateNewShip\n");
	fflush(fp);
	fclose(fp);
#endif

}



