/********************************************************************************************************\

	Mission:	Multiplayer Battlefest
	Script:		"Mul_3Battlefest"
	Descript:	Destroy your opponent 6 times (each time your opponent comes at you with a different ship).

	Created by:		Daniel J. Suleski	06/13/00
	Last Updated:	DJS					07/17/00
					Second Pass.
					DJS					08/23/00
					Map overrides.
					DJS					08/26/00
					Text check.  Removed mDisplayMessage() calls.  Disengage now destroys ships.
					DJS						09/26/00
					Meta script modifications

\********************************************************************************************************/
#include "MissionText.h"
#include "MissionMaps.h"
#include "Mul_3Battlefest.h"
#include "CommonTeam.h"
#include "VictoryCondition.h"
#include "WorldConditions.h"

#include "teamtracker.h"
#include "scriptstructs.h"

DEFINE_SCRIPT_CLASS( tMul_3Battlefest );

ShipList global_sl;
ShipList narrowed_sl;

#ifdef FIGHTERS
ShipList fighter_sl;
ShipList narrowed_fighter_sl;
#endif

int32 ShipListComplete = 0;
int32 AIsCreated = 0;

ShipList sl_array[12][kNumPlayableRaces][7];  // array of lists [teams][races][classes]
ShipList sl_array_backup[12][kNumPlayableRaces][7];

tMul_3Battlefest::tMul_3Battlefest()
{}

//---------------------------------------------------------------------------------------------
// returns mission specs used by Dynaverse to build campaigns
void tMul_3Battlefest::mScriptSpecs( tMissionSpec* Specs )
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "w");
	fprintf(fp, "Initializing bfest+.\n");
	fflush(fp);
	fclose(fp);
#endif

	Specs->fMissionTitle = Messages[kMissionTitle_msg];
	Specs->fMissionDescription = Messages[kDescription_msg];
	Specs->fHelpTags = Messages[kHelpTags_msg];
	Specs->fMissionType = kMultiplayerMission;
	Specs->fPlayerRace = kSpecAnyRace;
	Specs->fNumShips = kMissSpecNoShip;
	Specs->fMenuStates = kNoMenuButtons;
	Specs->fBandwidthPart = static_cast< eBandwidthPart >( k1Band | k5Client );
	Specs->fGenericFlags = kDisableCapture;

	const tTeamSpec& team1 = mCreateTeamSpec(	"Team 1",
												kTeam1,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team2 = mCreateTeamSpec(	"Team 2",
												kTeam2,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team3 = mCreateTeamSpec(	"Team 3",
												kTeam3,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team4 = mCreateTeamSpec(	"Team 4",
												kTeam4,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team5 = mCreateTeamSpec(	"Team 5",
												kTeam5,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team6 = mCreateTeamSpec(	"Team 6",
												kTeam6,
												kMaxChance,
												kFalse,
												kPlayableTeam,
												kMinChance,
												kSpecAnyRace,
												kAnyRace,
												typeid( tCommonTeam ) );

	const tTeamSpec& team7 = mCreateTeamSpec(	"NPC Ally Team 1",
												kTeam7,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagA);

	const tTeamSpec& team8 = mCreateTeamSpec(	"NPC Ally Team 2",
												kTeam8,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagB);

	const tTeamSpec& team9 = mCreateTeamSpec(	"NPC Ally Team 3",
												kTeam9,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagC);

	const tTeamSpec& team10 = mCreateTeamSpec(	"NPC Ally Team 4",
												kTeam10,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagD);

	const tTeamSpec& team11 = mCreateTeamSpec(	"NPC Ally Team 5",
												kTeam11,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagE);

	const tTeamSpec& team12 = mCreateTeamSpec(	"NPC Ally Team 6",
												kTeam12,
												kMaxChance,
												kFalse,
												kNPCTeam,
												kMinChance,
												kSpecPlayableRace,
												kAnyRace,
												typeid( tCommonTeam ),
												kTagF);

	Specs->mSetupTeam( team1 );
	Specs->mSetupTeam( team2 );
	Specs->mSetupTeam( team3 );
	Specs->mSetupTeam( team4 );
	Specs->mSetupTeam( team5 );
	Specs->mSetupTeam( team6 );

	Specs->mSetupTeam( team7 );
	Specs->mSetupTeam( team8 );
	Specs->mSetupTeam( team9 );
	Specs->mSetupTeam( team10 );
	Specs->mSetupTeam( team11 );
	Specs->mSetupTeam( team12 );
}

//---------------------------------------------------------------------------------------------
// called at the start of mission initialization - use to set prestige points 
// and override mission defaults such as:
//		allied race
//		enemy race
//		system name
// and also returns map index to use
int32 tMul_3Battlefest::mInitializeStart( tDynaverseScriptInfo& info )
{
	// pick a map
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mInitializeStart  (pick a map)\n");
	fflush(fp);
	fclose(fp);
#endif
	return 1;
}

//---------------------------------------------------------------------------------------------
// called to create ships before or during the mission
// Mode is user supplied value( expect for 0, which is initial setup call )
void tMul_3Battlefest::mCreateExternalShips( int32 Mode )
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mCreateExternalShips\n");
	fflush(fp);
	fclose(fp);
#endif
}

//---------------------------------------------------------------------------------------------
// called at the end of mission initialization - use to return briefing message
int32 tMul_3Battlefest::mInitializeFinished( void )
{
	// return the index to the briefing message we want to use
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mInitializeFinished\n");
	fflush(fp);
	fclose(fp);
#endif
	return( kBriefing_msg );
}

//---------------------------------------------------------------------------------------------
// called whenever a ship is created after a mission has already started
void tMul_3Battlefest::mNewInternalShip( tShipInfo* Ship )
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mNewInternalShip\n");
	fflush(fp);
	fclose(fp);
#endif
}

void tMul_3Battlefest::mStartMission()
{

#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mStartMission\n");
	fflush(fp);
	fclose(fp);
#endif

	tScript::mStartMission();

	const int kRelationsCount = 6;
	int32 tagList[kRelationsCount] =
	{
		kTagA,
		kTagB,
		kTagC,
		kTagD,
		kTagE,
		kTagF,
	};

	std::string relationships[kRelationsCount][kRelationsCount] =
	{
		//		 A     B     C     D     E     F
		/*A   */ "AI", "WD", "WD", "WD", "WD", "WD",
		/*B   */ "WD", "AI", "WD", "WD", "WD", "WD",
		/*C   */ "WD", "WD", "AI", "WD", "WD", "WD",
		/*D   */ "WD", "WD", "WD", "AI", "WD", "WD",
		/*E   */ "WD", "WD", "WD", "WD", "AI", "WD",
		/*F   */ "WD", "WD", "WD", "WD", "WD", "AI",
	};

	fMissionInfo->mSetTagRelations( tagList, ( std::string* )relationships, kRelationsCount );

	//
	//	The last thing that should be done before the mission actually starts is to check if the game
	//	has already been won.  This can most likely happen as a result of all teams having the same
	//	team tag.  Therefore, everyone is allied.
	//
	gWC.mCheckForGameEnd();

}

void tMul_3Battlefest::mNewTeam( const tTeamScriptDescription& teamScriptDescription )
{

#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mNewTeam -- Team %d\n", teamScriptDescription.fTeam);
	fflush(fp);
#endif

	switch ( teamScriptDescription.fTeamTag )
	{
	case kTagA:
	case kTagB:
	case kTagC:
	case kTagD:
	case kTagE:
	case kTagF:
		{
			// If the team is AI, skip! This will be handled in CommonTeam.cpp
			if ( teamScriptDescription.fTeam < 6 ) {
				mMission()->mSetupTeam(	teamScriptDescription.fTeam,
									teamScriptDescription.fTeamTag,
									teamScriptDescription.fRace,
									teamScriptDescription.fHumanControlled,
									typeid( tCommonTeam ) );
			}
		}
		break;
	}


#ifdef DEBUGOUT
	fprintf(fp, "Mul_3Battlefest::mNewTeam: Done setting up team %d.\n", teamScriptDescription.fTeam);
	fflush(fp);
	fclose(fp);
#endif
}

/*

	HELPER NOTE:	Make certain to call tScript::mASCIIOverrides() first before assigning custom values.

*/
void tMul_3Battlefest::mASCIIOverrides( tMapOverrides* Map )
{
#ifdef DEBUGOUT
	FILE *fp = fopen("C:\\bfest+assisted.txt", "a");
	fprintf(fp, "Entering: tMul_3Battlefest::mASCIIOverrides\n");
	fflush(fp);
	fclose(fp);
#endif

	tScript::mASCIIOverrides( Map );

	Map->fTacticalMapIcon['G'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['H'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['I'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['J'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['K'] = kObjectPlayerShip;
	Map->fTacticalMapIcon['L'] = kObjectPlayerShip;

	Map->fTacticalMapIcon['M'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['N'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['O'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['P'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['Q'] = kObjectEnemyShip;
	Map->fTacticalMapIcon['R'] = kObjectEnemyShip;

}
